#
# Copyright (c) 2014 Hyung-Suk Kim, Jorge Herrera, Blair Kaneshiro
# All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a 
# copy of this software and associated documentation files (the 
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#   
# The authors encourage users to include this copyright notice,
# and to let us know that you are using the Software Any person wishing to 
# distribute modifications to the Software is encouraged to send the 
# modifications to the original authors so that they can be incorporated 
# into the canonical version.
# 
# The Software is provided "as is", WITHOUT ANY WARRANTY, express or implied,
# including but not limited to the warranties of MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE and NONINFRINGEMENT.  In no event shall the authors
# or copyright holders by liable for any claim, damages, or other liability,
# whether in an actino of a contract, tort or otherwise, arising from, out of
# or in connection with the Software or the use or other dealings in the 
# software.
#

## loads query file
loadQueryFile = function(filename) {
  
  data = read.csv(filename, sep=" ", stringsAsFactors=FALSE, header=F)
  if(is.na(data[length(data)]) ) {
    data = data[1:(length(data)-1)] # remove last entry
  }
  
  return( unlist(data) )
}

## maps values to "s/u/d/" string
map2String = function(x = 1, threshold = 0.2 )
{
  if(abs(x) <= threshold) return('s')
  else if (x > 0) return('u')
  else return('d')
}

## returns the "s/u/d"-string for the duration
getDurationString = function(onset, release) {
  totalDuration = tail(release, 1)
  avgNoteDuration = totalDuration/length(onset)
  
  durations = append(onset, totalDuration)
  normDuration = diff(durations)/avgNoteDuration
  dd = diff(normDuration)
  
  return( paste(sapply(dd, map2String), collapse="") )
}

## returns the "s/u/d"-string for melody contours
getMelString = function(notes, tr=0.1) {
  diffY = diff(notes)
  normDiffY = diffY/max(abs(diffY))
  
  return( paste(sapply(normDiffY, map2String, threshold=tr), collapse="") )
}

## compares the query q to the dictionary dict
## and returns the results by score order
## returns a list by score
getScores = function(q, dict) {
  d = adist(q, dict)
  
  scores = d[order(d)]
  songTitles = colnames(d)[order(d)]
  
  names(scores) = songTitles
  
  return(scores)
}

## returns the Euclidean distance 
mergeDistances = function(dur, mel, pow=2, weights=c(0.5, 0.5)) {
  weights = weights/sum(weights)
  
  dd = dur[ order(names(dur)) ]
  mm = mel[ order(names(mel)) ]
  
  merged = weights[1]*dd^pow + weights[2]*mm^pow
  
  names(merged) = names(dur)[order(names(dur))]
  
  return( merged[order(merged)] )
}

## trims leading
trim.leading <- function (x)  sub("^\\s+", "", x)